#!/usr/bin/env python
# coding: utf-8

from starlette.requests import Request

import ray
from ray import serve
import os
import numpy as np

# download data
minio_endpoint = "http://210.110.233.67:9005"
bucket = "public"
file_path = "spherex-qc/zodicube2_4x6_21Mar.npy"
fn = os.path.join("data", os.path.basename(file_path))
url = f"{minio_endpoint}/{bucket}/{file_path}"
url, fn

if not os.path.exists(fn):
    print(f"downloading {url} -> {fn}")
    os.system(f"curl -o {fn} {url}")

print("file downloaded")

runtime_env = {"working_dir": ".", "excludes": ["anaconda3", "data", "__pycache__"]}
print(runtime_env)

ray.init("auto", runtime_env=runtime_env)

# Load the data and put it on the shared memory
fn = "data/zodicube2_4x6_21Mar.npy"
data = np.load(fn)
zodi_data = dict((int(k), data[k]) for k in "123456")

r_zodi_data = ray.put(zodi_data)

ray_actor_options = dict()

n_spherex_resources = len([node for node in ray.nodes() if "spherex" in node["Resources"]])
if n_spherex_resources:
    ray_actor_options["resources"] = dict(spherex=1)
print(ray_actor_options)


@serve.deployment(ray_actor_options=ray_actor_options, num_replicas=3)
class SPHERExQCDeployment:
    "Derived class to suuport ray.get for data loading"
    def __init__(self, r_zodi_data):
        from spherex_qc import SPHERExQCHandler as _SPHERExQCHandler
        zodi_data = ray.get(r_zodi_data)
        self.handler = _SPHERExQCHandler(zodi_data)

    def query(self, name, ra, dec, zodi=False):
        return self.handler.query(name, ra, dec, zodi)

#SPHERExQCDeployment = serve.deployment(SPHERExQCHandler)
SPHERExQCDeployment.deploy(r_zodi_data)


qc_handle = SPHERExQCDeployment.get_handle()


releases_info = {"releases": []}

api_root = "/api/qc/v1"
RELEASENAME = "kasi20211021pre01"
RELEASEDATE = "20211012"
api_prefix = f"{api_root}/release/{RELEASENAME}/query"

releases_info["releases"].append(dict(release_name="kasi20211021pre01",
                                      release_date="2021-10-21",
                                      api_prefix=api_prefix))

# the immediate results is an astropy table, so we need this actor on spherex=1
@serve.deployment(route_prefix=api_prefix, ray_actor_options=ray_actor_options)
class ReleaseHandler:
    def __init__(self, qc_handle):
        self.qc_handle = qc_handle

    async def __call__(self, request: Request):
        ra = request.query_params.get("ra", None)
        dec = request.query_params.get("dec", None)

        name = request.query_params.get("name", "")
        zodi = request.query_params.get("zodi", False)

        tbl = await qc_handle.query.remote(name, ra, dec, zodi=zodi)

        d = tbl.to_pandas().to_dict("split")
        return d


ReleaseHandler.deploy(qc_handle)


@serve.deployment(route_prefix=f"{api_root}/releases")
class Releases:
    def __init__(self, releases_info):
        self.releases_info = releases_info

    def __call__(self, request: Request):
        return self.releases_info


Releases.deploy(releases_info)

