from starlette.requests import Request
import numpy as np
import pandas as pd

import ray
from ray import serve
from spherex_qc import SPHERExQCHandler as _SPHERExQCHandler

def get_url():
    import os
    minio_endpoint = "http://210.110.233.67:9005"
    bucket = "public"
    file_path = "spherex-qc/zodicube2_4x6_21Mar.npy"
    # fn = os.path.join("data", os.path.basename(file_path))
    url = f"{minio_endpoint}/{bucket}/{file_path}"

    return url


@serve.deployment(num_replicas=1, ray_actor_options={"num_cpus": 0.2,
                                                     "num_gpus": 0})
class SPHERExQCDataLoader:
    def __init__(self):
        # fn = "../data/zodicube2_4x6_21Mar.npy"
        # data = np.load(fn)

        import requests
        import io

        url = get_url()
        response = requests.get(url)
        response.raise_for_status()
        data = np.load(io.BytesIO(response.content))

        self.data_ref = ray.put(data)

    def get_data_ref(self):
        return self.data_ref


# single instance takes 1.34GB of memory
@serve.deployment(num_replicas=1, ray_actor_options={"num_cpus": 0.2,
                                                     "num_gpus": 0})
class SPHERExQCDeployment:
    async def __init__(self, data_loader_bind):
        # if zodi_data_ref is None:
        #     # fn = "../data/zodicube2_4x6_21Mar.npy"
        #     # data = np.load(fn)
        data_ref_ref = await data_loader_bind.get_data_ref.remote()
        data_ref = ray.get(data_ref_ref)
        data = ray.get(data_ref)
        zodi_data = dict((int(k), data[k]) for k in "123456")
        # else:
        #     data = ray.get(zodi_data_ref)
        #     zodi_data = dict((int(k), data[k]) for k in "123456")

        self.handler = _SPHERExQCHandler(zodi_data)

    # def get_data_ref_type(self):
    #     return self.ddd

    def query(self, name: str, ra: float, dec: float, zodi: bool=False) -> pd.DataFrame:
        return self.handler.query(name, ra, dec, zodi).to_pandas()

    async def __call__(self, http_request: Request) -> pd.DataFrame:
        # query_params = await http_request.json()
        query_params = http_request.query_params  # await http_request.json()

        ra = query_params.get("ra", None)
        dec = query_params.get("dec", None)

        name = query_params.get("name", "")
        zodi = query_params.get("zodi", False)

        df = self.query(name, ra, dec, zodi=zodi)

        return df

        # d = tbl.to_pandas().to_dict("split")

# ray.init("auto")

# zodi_data_ref = ray.put(np.arange(100))

# deployment = SPHERExQCDeployment.bind(SPHERExQCZodiLoader.bind())
# zodi_data = ray.get(zodi_data_ref)
def main():
    if False:
        fn = "../data/zodicube2_4x6_21Mar.npy"
        data = np.load(fn)
        zodi_data = data
        zodi_data_ref = ray.put(zodi_data)

        # deployment = SPHERExQCDeployment.bind(zodi_data_ref)
    data_loader_bind = SPHERExQCDataLoader.bind()
    deployment = SPHERExQCDeployment.bind(data_loader_bind)
    # Using "serve run" CLI command with this script fails, because of the "ray.put" statement. So, we use serve.run from python.
    handle = serve.run(deployment)
    # data = ray.get(handle.get_data_ref_type.remote())
    # print(type(data))


if __name__ == '__main__':
    main()
else:
    data_loader_bind = SPHERExQCDataLoader.bind()
    deployment = SPHERExQCDeployment.bind(data_loader_bind)
