import os
import numpy as np
import astropy.units as u
from astropy.coordinates import SkyCoord, ICRS

import SPHEREx_ObsSimulator as SPobs
import SPHEREx_InstrumentSimulator as SPinst
import SPHEREx_SkySimulator as SPsky

from SPHEREx_SkySimulator.quicker_catalog import (
    QuickerCatalogFromZodiCube
)

from astropy.coordinates import SkyCoord
# from SPHEREx_SkySimulator import Catalog_to_Simulate as _Catalog_to_Simulate

from SPHEREx_Simulator_Tools import data_filename
# from scipy import interpolate



class SPHERExQCHandler:
    @staticmethod
    def get_spherex_default_setting():
        # Get a SPHEREx Survey Plan
        survey_plan_file = data_filename("spherex_survey_plan_march_2021.fits")
        psf_file = data_filename("psf/simulated_PSF_database.fits")

        SPHEREx_Pointings = SPobs.Pointings(input_file=survey_plan_file,
                                            Gaussian_jitter = 0.0,
                                            roll_angle='psi2')
        # Get an SPHEREx Instrument
        SPHEREx_Instrument = SPinst.Instrument(
            psf=psf_file,
            psf_downsample_by_array = {1:4, 2:4, 3:4, 4:2, 5:2, 6:2},
            psf_trim_by_array = {1:32, 2:32, 3:32, 4:32, 5:32, 6:32},
            noise_model = SPinst.white_noise,
            dark_current_model = SPinst.poisson_dark_current,
            lvf_model = SPinst.smile_lvf)

        # Set up a sky scene in order to initialize a background model
        Scene = SPsky.Scene(SPHEREx_Pointings,
                            zodi_model=SPsky.zodicalc.ModifiedKelsallModelWithHPFT())

        return dict(pointing=SPHEREx_Pointings,
                    inst=SPHEREx_Instrument,
                    scene=Scene)

    def __init__(self, zodi_data, survey_plan_file=None):

        if survey_plan_file is None:
            survey_plan_file = data_filename("spherex_survey_plan_march_2021.fits")

        # psf_file = data_filename('psf/simulated_PSF_database.fits')

        # self.pointings = SPobs.Pointings(input_file=survey_plan_file,
        #                                  Gaussian_jitter = 0.0,
        #                                  roll_angle='psi2')
        # # Get an SPHEREx Instrument
        # self.inst = SPinst.Instrument(psf=psf_file,
        #                               psf_downsample_by_array = {1:4, 2:4, 3:4, 4:2, 5:2, 6:2},
        #                               psf_trim_by_array = {1:32, 2:32, 3:32, 4:32, 5:32, 6:32},
        #                               noise_model = SPinst.white_noise,
        #                               dark_current_model = SPinst.poisson_dark_current,
        #                               lvf_model = SPinst.smile_lvf)

        ss = self.get_spherex_default_setting()
        zodi_model = ss['scene'].zodi_model

        self.qc = QuickerCatalogFromZodiCube(ss["pointing"],
                                             ss["inst"],
                                             zodi_model._itp_solspect,
                                             zodi_model._itp_zlref,
                                             zodi_data)

        # self.qc = QuickerCatalogFromZodiCube(
        #     self.pointings, self.inst,
        #     zodi_cube_file=zodi_cube_file
        # )

    def query(self, name, ra, dec, zodi=False):
        # name, ra, dec = "M31", 10.68471, 41.26875
        c = SkyCoord(ra, dec, unit="degree", frame=ICRS())

        tbl = self.qc.find_obs(name, c)

        if zodi:
            tbl["ZODI"] = self.qc.get_zodi(tbl)

        return tbl

    def get_zodi(self, tbl):
        return self.qc.get_zodi(tbl)


def main():

    zodi_cube_file = "../data/zodicube2_4x6_21Mar.npy"

    data = np.load(zodi_cube_file)

    zodi_data = dict((int(k), data[k]) for k in "123456")
    handler = SPHERExQCHandler(zodi_data)
    k = handler.query('test', 10.68, 41.2)
    kz = handler.query('test', 10.68, 41.2, zodi=True)

if False:
    # fn = "zodicube2_4x6_21Mar.npy"
    # fn = "/home/jjlee/work_active/spherex/SPHEREx-Sky-Simulator/docs/QuickCatalog/zodicube_5x5_21Mar.npy"

    fn = "zodicube2_4x6_21Mar.npy"
    data = np.load(fn)
    zodi_data = dict((int(k), data[k]) for k in "123456")

    handler = SPHERExQCHandler(zodi_data)
    k = handler.query('test', 10.68, 41.2)
    kz = handler.query('test', 10.68, 41.2, zodi=True)

